@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,15 +55,21 @@ import java.io.UnsupportedEncodingException;
 import java.net.InetAddress;
 import java.net.SocketTimeoutException;
 import java.net.UnknownHostException;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
 import java.util.concurrent.ConcurrentHashMap;
 import javax.mail.AuthenticationFailedException;
+import javax.mail.Message;
 import javax.mail.MessagingException;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mail.mime.MIMESessionPropertyNames;
+import com.openexchange.mail.mime.utils.MIMEStorageUtility;
 import com.openexchange.pop3.POP3Exception;
 import com.openexchange.pop3.POP3Provider;
 import com.openexchange.pop3.config.IPOP3Properties;
@@ -73,6 +79,8 @@ import com.openexchange.pop3.config.POP3SessionProperties;
 import com.openexchange.pop3.util.POP3CapabilityCache;
 import com.openexchange.session.Session;
 import com.openexchange.tools.ssl.TrustAllSSLSocketFactory;
+import com.sun.mail.pop3.POP3Folder;
+import com.sun.mail.pop3.POP3Message;
 import com.sun.mail.pop3.POP3Store;
 
 /**
@@ -84,6 +92,79 @@ public final class POP3StoreConnector {
 
     private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3StoreConnector.class);
 
+    /**
+     * The result after establishing a connection to POP3 server.
+     */
+    public static final class POP3StoreResult {
+
+        private final String capabilities;
+
+        private POP3Store pop3Store;
+
+        private final List<MailException> warnings;
+
+        protected POP3StoreResult(final String capabilities) {
+            super();
+            this.capabilities = capabilities;
+            warnings = new ArrayList<MailException>(2);
+        }
+
+        /**
+         * Sets the connected {@link POP3Store} instance.
+         * 
+         * @param pop3Store The connected {@link POP3Store} instance.
+         */
+        protected void setPop3Store(final POP3Store pop3Store) {
+            this.pop3Store = pop3Store;
+        }
+
+        /**
+         * Adds given warnings.
+         * 
+         * @param warning The warning to add
+         */
+        protected void addWarning(final MailException warning) {
+            warnings.add(warning);
+        }
+
+        /**
+         * Gets the warnings occurred during establishing a connection to POP3 server.
+         * 
+         * @return The warnings
+         */
+        public Collection<MailException> getWarnings() {
+            return Collections.unmodifiableCollection(warnings);
+        }
+
+        /**
+         * Gets the connected {@link POP3Store} instance.
+         * 
+         * @return The connected {@link POP3Store} instance.
+         */
+        public POP3Store getPop3Store() {
+            return pop3Store;
+        }
+
+        /**
+         * Gets the POP3 server's capabilities.
+         * 
+         * @return The POP3 server's capabilities.
+         */
+        public String getCapabilities() {
+            return capabilities;
+        }
+
+        /**
+         * Checks if this result contains one or more warnings.
+         * 
+         * @return <code>true</code> if this result contains one or more warnings; otherwsie <code>false</code>
+         */
+        public boolean containsWarnings() {
+            return !warnings.isEmpty();
+        }
+
+    }
+
     private static Map<HostAndPort, Long> timedOutServers;
 
     private static Map<LoginAndPass, Long> failedAuths;
@@ -118,10 +199,11 @@ public final class POP3StoreConnector {
      * @param pop3Properties Optional additional POP3 properties applied to POP3 session (may be <code>null</code>)
      * @param monitorFailedAuthentication <code>true</code> to monitor failed authentication; otherwise <code>false</code>
      * @param session The session providing user information
+     * @param errorOnMissingUIDL <code>true</code> to throw an error on missing UIDL; otherwise <code>false</code> to ignore
      * @return A connected instance of {@link POP3Store}
      * @throws MailException If establishing a connected instance of {@link POP3Store} fails
      */
-    public static POP3Store getPOP3Store(final POP3Config pop3Config, final Properties pop3Properties, final boolean monitorFailedAuthentication, final Session session) throws MailException {
+    public static POP3StoreResult getPOP3Store(final POP3Config pop3Config, final Properties pop3Properties, final boolean monitorFailedAuthentication, final Session session, final boolean errorOnMissingUIDL) throws MailException {
         try {
             final boolean tmpDownEnabled = (POP3Properties.getInstance().getPOP3TemporaryDown() > 0);
             if (tmpDownEnabled) {
@@ -137,11 +219,12 @@ public final class POP3StoreConnector {
             final int port = pop3Config.getPort();
             final String capabilities;
             try {
-                capabilities = POP3CapabilityCache.getCapability(
-                    InetAddress.getByName(server),
-                    port,
-                    pop3Config.isSecure(),
-                    (IPOP3Properties) pop3Config.getMailProperties());
+                capabilities =
+                    POP3CapabilityCache.getCapability(
+                        InetAddress.getByName(server),
+                        port,
+                        pop3Config.isSecure(),
+                        (IPOP3Properties) pop3Config.getMailProperties());
             } catch (final UnknownHostException e) {
                 throw new MessagingException(e.getMessage(), e);
             } catch (final IOException e) {
@@ -150,24 +233,8 @@ public final class POP3StoreConnector {
             /*
              * JavaMail POP3 implementation requires capabilities "UIDL" and "TOP"
              */
+            final POP3StoreResult result = new POP3StoreResult(capabilities);
             final String login = pop3Config.getLogin();
-            if (capabilities.indexOf("TOP") < 0) {
-                throw new POP3Exception(
-                    POP3Exception.Code.MISSING_REQUIRED_CAPABILITY,
-                    "TOP",
-                    server,
-                    login,
-                    Integer.valueOf(session.getUserId()),
-                    Integer.valueOf(session.getContextId()));
-            } else if (capabilities.indexOf("UIDL") < 0) {
-                throw new POP3Exception(
-                    POP3Exception.Code.MISSING_REQUIRED_CAPABILITY,
-                    "UIDL",
-                    server,
-                    login,
-                    Integer.valueOf(session.getUserId()),
-                    Integer.valueOf(session.getContextId()));
-            }
             final boolean responseCodeAware = capabilities.indexOf("RESP-CODES") >= 0;
             String tmpPass = pop3Config.getPassword();
             if (tmpPass != null) {
@@ -269,12 +336,88 @@ public final class POP3StoreConnector {
                 }
                 throw e;
             }
-            return pop3Store;
+            /*
+             * Check for needed capabilities
+             */
+            final boolean hasTop = (capabilities.indexOf("TOP") >= 0);
+            final boolean hasUidl = (capabilities.indexOf("UIDL") >= 0);
+            if (!hasTop || !hasUidl) {
+                final POP3Folder inbox = (POP3Folder) pop3Store.getFolder("INBOX");
+                inbox.open(POP3Folder.READ_ONLY);
+                try {
+                    final Message[] allMessages = inbox.getMessages();
+                    if (allMessages.length > 0) {
+                        final Message[] single = new Message[1];
+                        single[0] = allMessages[0];
+                        probeUIDL(session, server, result, login, hasUidl, inbox, single, errorOnMissingUIDL);
+                        probeTOP(session, server, login, hasTop, single);
+                    }
+                } finally {
+                    inbox.close(false);
+                }
+            }
+            result.setPop3Store(pop3Store);
+            return result;
         } catch (final MessagingException e) {
             throw MIMEMailException.handleMessagingException(e, pop3Config, session);
         }
     }
 
+    private static void probeTOP(final Session session, final String server, final String login, final boolean hasTop, final Message[] single) throws POP3Exception {
+        if (!hasTop) {
+            /*
+             * Probe TOP
+             */
+            try {
+                ((POP3Message) single[0]).top(1);
+            } catch (final Exception e) {
+                /*-
+                 * Probe failed.
+                 * Mandatory to further work with JavaMail API
+                 */
+                throw new POP3Exception(
+                    POP3Exception.Code.MISSING_REQUIRED_CAPABILITY,
+                    "TOP",
+                    server,
+                    login,
+                    Integer.valueOf(session.getUserId()),
+                    Integer.valueOf(session.getContextId()));
+            }
+        }
+    }
+
+    private static void probeUIDL(final Session session, final String server, final POP3StoreResult result, final String login, final boolean hasUidl, final POP3Folder inbox, final Message[] single, final boolean errorOnMissingUIDL) throws POP3Exception {
+        if (!hasUidl) {
+            /*
+             * Probe UIDL
+             */
+            try {
+                inbox.fetch(single, MIMEStorageUtility.getUIDFetchProfile());
+            } catch (final Exception e) {
+                /*-
+                 * Probe failed.
+                 * Avoid fetching UIDs when further working with JavaMail API
+                 */
+                if (errorOnMissingUIDL) {
+                    throw new POP3Exception(
+                        POP3Exception.Code.MISSING_REQUIRED_CAPABILITY,
+                        "UIDL",
+                        server,
+                        login,
+                        Integer.valueOf(session.getUserId()),
+                        Integer.valueOf(session.getContextId()));
+                }
+                result.addWarning(new POP3Exception(
+                    POP3Exception.Code.EXPUNGE_MODE_ONLY,
+                    "UIDL",
+                    server,
+                    login,
+                    Integer.valueOf(session.getUserId()),
+                    Integer.valueOf(session.getContextId())));
+            }
+        }
+    }
+
     private static void checkFailedAuths(final String login, final String pass) throws AuthenticationFailedException {
         final LoginAndPass key = new LoginAndPass(login, pass);
         final Long range = failedAuths.get(key);
