@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.pop3.storage;
 
+import java.util.Collection;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.api.IMailFolderStorage;
 import com.openexchange.mail.api.IMailMessageStorage;
@@ -74,6 +75,13 @@ public interface POP3Storage {
      */
     public void close() throws MailException;
 
+    /**
+     * Gets possible warnings.
+     * 
+     * @return Possible warnings.
+     */
+    public Collection<MailException> getWarnings();
+
     /**
      * Convenience method to obtain folder's number of unread messages in a fast way; meaning no default folder check is performed.
      * 
