@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -73,7 +73,15 @@ import com.openexchange.tools.ssl.TrustAllSSLSocketFactory;
  */
 public final class POP3CapabilityCache {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3CapabilityCache.class);
+    /**
+     * The logger.
+     */
+    static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(POP3CapabilityCache.class);
+
+    /**
+     * The default capabilities providing only mandatory POP3 commands.
+     */
+    static final Capability DEFAULT_CAPABILITIES = new Capability("USER\r\nPASS\r\nSTAT\r\nLIST\r\nRETR\r\nDELE\r\nNOOP\r\nRSET\r\nQUIT");
 
     private static ConcurrentMap<InetSocketAddress, Future<Capability>> MAP;
 
@@ -181,11 +189,8 @@ public final class POP3CapabilityCache {
     public static String getCapability(final InetSocketAddress address, final boolean isSecure, final int connectionTimeout, final int timeout) throws IOException {
         Future<Capability> f = MAP.get(address);
         if (null == f) {
-            final FutureTask<Capability> ft = new FutureTask<Capability>(new CapabilityCallable(
-                address,
-                isSecure,
-                connectionTimeout,
-                timeout));
+            final FutureTask<Capability> ft =
+                new FutureTask<Capability>(new CapabilityCallable(address, isSecure, connectionTimeout, timeout));
             f = MAP.putIfAbsent(address, ft);
             if (null == f) {
                 f = ft;
@@ -353,7 +358,13 @@ public final class POP3CapabilityCache {
                                 sb.append(c);
                             }
                         }
-                        throw new IOException("POP3 CAPA command failed: " + sb.toString());
+                        /*
+                         * Assume default capabilities: USER, PASS, STAT, LIST, RETR, DELE, NOOP, RSET, QUIT
+                         */
+                        if (LOG.isWarnEnabled()) {
+                            LOG.warn(sb.insert(0, "POP3 CAPA command failed: ").toString());
+                        }
+                        return DEFAULT_CAPABILITIES;
                     } else if ('+' == pre) {
                         sb.append(pre);
                         eol = false;
@@ -372,7 +383,10 @@ public final class POP3CapabilityCache {
                         }
                         final String responseCode = sb.toString();
                         if (!responseCode.toUpperCase().startsWith("+OK")) {
-                            throw new IOException("POP3 CAPA command failed: " + responseCode);
+                            if (LOG.isWarnEnabled()) {
+                                LOG.warn(new StringBuilder("POP3 CAPA command failed: ").append(responseCode).toString());
+                            }
+                            return DEFAULT_CAPABILITIES;
                         }
                         sb.setLength(0);
                         if (skipLF) {
@@ -405,7 +419,14 @@ public final class POP3CapabilityCache {
                         }
                         capabilities = sb.toString();
                     } else {
-                        throw new IOException("Unexpected response start: " + pre);
+                        if (LOG.isWarnEnabled()) {
+                            if (Character.isDefined(pre)) {
+                                LOG.warn(new StringBuilder("Unexpected CAPA response start: ").append(pre).toString());
+                            } else {
+                                LOG.warn(new StringBuilder("Invalid unicode character: ").append(((int) pre)).toString());
+                            }
+                        }
+                        return DEFAULT_CAPABILITIES;
                     }
                 }
                 /*
